@@ -1,5 +1,5 @@
 /* Assembler macros for CRIS.
-   Copyright (C) 1999, 2001 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2004 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -169,9 +169,13 @@
 #define ASM_ARGS_0() "r" (__callno)
 #define CHECK_ARGS_asm_0
 
+/* When assigning to an asm-declared register, make sure no function call
+   can come between that and its use.  Hence the extra indirection, using
+   temporary variables.  FIXME: Bug or undocumented beavior in GCC.  */
 #define LOAD_ARGS_c_1(r10) \
+	unsigned long __r10_ = (unsigned long) (r10); 		\
 	LOAD_ARGS_c_0()						\
-	register unsigned long __r10 __asm__ ("r10") = (unsigned long) (r10);
+	register unsigned long __r10 __asm__ ("r10") = __r10_;
 #define LOAD_ARGS_asm_1(r10) LOAD_ARGS_asm_0 ()
 #define ASM_CLOBBER_1 ASM_CLOBBER_0
 #define ASM_ARGS_1(r10) ASM_ARGS_0 (), "0" (__r10)
@@ -181,8 +185,9 @@
 	".endif\n\t"
 
 #define LOAD_ARGS_c_2(r10, r11) \
+	unsigned long __r11_ = (unsigned long) (r11); 		\
 	LOAD_ARGS_c_1(r10)					\
-	register unsigned long __r11 __asm__ ("r11") = (unsigned long) (r11);
+	register unsigned long __r11 __asm__ ("r11") = __r11_;
 #define LOAD_ARGS_asm_2(r10, r11) LOAD_ARGS_asm_1 (r10)
 #define ASM_CLOBBER_2 ASM_CLOBBER_1
 #define ASM_ARGS_2(r10, r11) ASM_ARGS_1 (r10), "r" (__r11)
@@ -192,8 +197,9 @@
 	".endif\n\t"
 
 #define LOAD_ARGS_c_3(r10, r11, r12) \
+	unsigned long __r12_ = (unsigned long) (r12);		\
 	LOAD_ARGS_c_2(r10, r11)					\
-	register unsigned long __r12 __asm__ ("r12") = (unsigned long) (r12);
+	register unsigned long __r12 __asm__ ("r12") = __r12_;
 #define LOAD_ARGS_asm_3(r10, r11, r12) LOAD_ARGS_asm_2 (r10, r11)
 #define ASM_CLOBBER_3 ASM_CLOBBER_2
 #define ASM_ARGS_3(r10, r11, r12) ASM_ARGS_2 (r10, r11), "r" (__r12)
@@ -203,8 +209,9 @@
 	".endif\n\t"
 
 #define LOAD_ARGS_c_4(r10, r11, r12, r13) \
+	unsigned long __r13_ = (unsigned long) (r13);		\
 	LOAD_ARGS_c_3(r10, r11, r12)				\
-	register unsigned long __r13 __asm__ ("r13") = (unsigned long) (r13);
+	register unsigned long __r13 __asm__ ("r13") = __r13_;
 #define LOAD_ARGS_asm_4(r10, r11, r12, r13) LOAD_ARGS_asm_3 (r10, r11, r12)
 #define ASM_CLOBBER_4 ASM_CLOBBER_3
 #define ASM_ARGS_4(r10, r11, r12, r13) ASM_ARGS_3 (r10, r11, r12), "r" (__r13)
